/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Martin Rexa
 */

package cz.xelfi.quoridor.statistics;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.Collections;
import java.util.ArrayList;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
/**
 *
 * @author Martin Rexa
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public class EloList {
    static int INITIAL_ELO = 1250;
    static int MAX_ELO_DIFF = 400;
    static int KVAL_START = 25;
    static int KVAL_FOLLOW = 10;
    static int KVAL_LIMIT = 30;

    Map<String, Double> players;
    Map<String, Integer> playerGames;
    long published = 0;
    boolean originalList;


    public EloList () {
        players = new HashMap<String, Double>();
        playerGames = new HashMap<String, Integer>();
        originalList = true;
    }

    public EloList(EloList src){
        this.players = new HashMap<String, Double>(src.players);
        this.playerGames = new HashMap<String, Integer>(src.playerGames);
        originalList = false;
    }

    public void setEndDate(long endDate){
        this.published = endDate;
    }

    public long getEndDate(){
        return this.published;
    }

    @XmlElementWrapper(name="elolist")
    @XmlElement(name="item")
    public List<EloEntry> getFinalList(){
        List<EloEntry> finalList = new ArrayList<EloEntry>();
        for (Map.Entry<String, Double> player : players.entrySet()){
            finalList.add(new EloEntry(player.getKey(), player.getValue(), playerGames.get(player.getKey())));
        }
        Collections.sort(finalList, EloEntry.BEST_FIRST);
        return finalList;
    }

    public EloList putResult(String winner, String looser){
        double wElo = getElo(winner);
        double lElo = getElo(looser);

        double eloDiff = wElo - lElo;
        if(eloDiff < - MAX_ELO_DIFF)
            eloDiff = - MAX_ELO_DIFF;
        if(eloDiff > MAX_ELO_DIFF)
            eloDiff = MAX_ELO_DIFF;

        double wDiff = Math.round(100 * getKVal(winner) * (1 - (1 / (1 + Math.pow(10, -eloDiff / MAX_ELO_DIFF)))));
        wDiff /= 100.0;
        double lDiff = Math.round(100 * getKVal(looser) * (0 - (1 / (1 + Math.pow(10, eloDiff / MAX_ELO_DIFF)))));
        lDiff /= 100.0;

        players.put(winner, wElo + wDiff);
        players.put(looser, lElo + lDiff);
        return this;
    }
    
    public double getElo(String player){
        Double elo = players.get(player);
        return (elo == null) ? INITIAL_ELO : elo;
    }
    
    private int getKVal(String player){
        Integer games = playerGames.get(player);
        int g = (games == null) ? 0 : games;
        // Side effect, method should not be called more times for the same game/player
        playerGames.put(player, g + 1);
        return g < KVAL_LIMIT ? KVAL_START : KVAL_FOLLOW;
    }

}
